<?php
namespace Ak\Mailchimp\Entity;

use DateTime;

/**
 * Mailing lista.
 *
 * @package Ak\Mailchimp\Entity
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/autokreso/mailchimp
 */
class MailingList
{
    /** @var string */
    private $id;
    /** @var int */
    private $web_id;
    /** @var string */
    private $name;
    /** @var MailingList\Contact */
    private $contact;
    /** @var string */
    private $permission_reminder;
    /** @var bool */
    private $use_archive_bar;
    /** @var MailingList\CampaignDefaults */
    private $campaign_defaults;
    /** @var string */
    private $notify_on_subscribe;
    /** @var string */
    private $notify_on_unsubscribe;
    /** @var DateTime */
    private $date_created;
    /** @var int */
    private $list_rating;
    /** @var bool */
    private $email_type_option;
    /** @var string */
    private $subscribe_url_short;
    /** @var string */
    private $subscribe_url_long;
    /** @var string */
    private $beamer_address;
    /** @var string */
    private $visibility;
    /** @var bool */
    private $double_optin;
    /** @var bool */
    private $has_welcome;
    /** @var bool */
    private $marketing_permissions;
    /** @var string[] */
    private $modules;
    /** @var MailingList\Stats */
    private $stats;

    /**
     * @return string
     */
    public function getId(): string
    {
        return $this->id;
    }

    /**
     * @return int
     */
    public function getWebId(): int
    {
        return $this->web_id ?? 0;
    }

    /**
     * @return string
     */
    public function getName(): string
    {
        return $this->name ?? '';
    }

    /**
     * @param string $name
     */
    public function setName(string $name): void
    {
        $this->name = $name;
    }

    /**
     * @return MailingList\Contact
     */
    public function getContact(): MailingList\Contact
    {
        if (null === $this->contact) {
            $this->contact = new MailingList\Contact();
        }

        return $this->contact;
    }

    /**
     * @param MailingList\Contact $contact
     */
    public function setContact(MailingList\Contact $contact): void
    {
        $this->contact = $contact;
    }

    /**
     * @return string
     */
    public function getPermissionReminder(): string
    {
        return $this->permission_reminder ?? '';
    }

    /**
     * @param string $permission_reminder
     */
    public function setPermissionReminder(string $permission_reminder): void
    {
        $this->permission_reminder = $permission_reminder;
    }

    /**
     * @return bool
     */
    public function getUseArchiveBar(): bool
    {
        return $this->use_archive_bar ?? false;
    }

    /**
     * @param bool $use_archive_bar
     */
    public function setUseArchiveBar(bool $use_archive_bar): void
    {
        $this->use_archive_bar = $use_archive_bar;
    }

    /**
     * @return MailingList\CampaignDefaults
     */
    public function getCampaignDefaults(): MailingList\CampaignDefaults
    {
        if (null === $this->campaign_defaults) {
            $this->campaign_defaults = new MailingList\CampaignDefaults();
        }

        return $this->campaign_defaults;
    }

    /**
     * @param MailingList\CampaignDefaults $campaign_defaults
     */
    public function setCampaignDefaults(MailingList\CampaignDefaults $campaign_defaults): void
    {
        $this->campaign_defaults = $campaign_defaults;
    }

    /**
     * @return string
     */
    public function getNotifyOnSubscribe(): string
    {
        return $this->notify_on_subscribe ?? '';
    }

    /**
     * @param string $notify_on_subscribe
     */
    public function setNotifyOnSubscribe(string $notify_on_subscribe): void
    {
        $this->notify_on_subscribe = $notify_on_subscribe;
    }

    /**
     * @return string
     */
    public function getNotifyOnUnsubscribe(): string
    {
        return $this->notify_on_unsubscribe ?? '';
    }

    /**
     * @param string $notify_on_unsubscribe
     */
    public function setNotifyOnUnsubscribe(string $notify_on_unsubscribe): void
    {
        $this->notify_on_unsubscribe = $notify_on_unsubscribe;
    }

    /**
     * @return DateTime
     */
    public function getDateCreated(): DateTime
    {
        return $this->date_created ?? DateTime::createFromFormat(DateTime::ISO8601, '0000-00-00T00:00:00+0000');
    }

    /**
     * @return int
     */
    public function getListRating(): int
    {
        return $this->list_rating ?? 0;
    }

    /**
     * @return bool
     */
    public function getEmailTypeOption(): bool
    {
        return $this->email_type_option ?? false;
    }

    /**
     * @param bool $email_type_option
     */
    public function setEmailTypeOption(bool $email_type_option): void
    {
        $this->email_type_option = $email_type_option;
    }

    /**
     * @return string
     */
    public function getSubscribeUrlShort(): string
    {
        return $this->subscribe_url_short ?? '';
    }

    /**
     * @return string
     */
    public function getSubscribeUrlLong(): string
    {
        return $this->subscribe_url_long ?? '';
    }

    /**
     * @return string
     */
    public function getBeamerAddress(): string
    {
        return $this->beamer_address ?? '';
    }

    /**
     * @return string
     */
    public function getVisibility(): string
    {
        return $this->visibility ?? '';
    }

    /**
     * @param string $visibility
     */
    public function setVisibility(string $visibility): void
    {
        $this->visibility = $visibility;
    }

    /**
     * @return bool
     */
    public function getDoubleOptin(): bool
    {
        return $this->double_optin ?? false;
    }

    /**
     * @param bool $double_optin
     */
    public function setDoubleOptin(bool $double_optin): void
    {
        $this->double_optin = $double_optin;
    }

    /**
     * @return bool
     */
    public function getHasWelcome(): bool
    {
        return $this->has_welcome ?? false;
    }

    /**
     * @param bool $has_welcome
     */
    public function setHasWelcome(bool $has_welcome): void
    {
        $this->has_welcome = $has_welcome;
    }

    /**
     * @return bool
     */
    public function getMarketingPermissions(): bool
    {
        return $this->marketing_permissions ?? false;
    }

    /**
     * @param bool $marketing_permissions
     */
    public function setMarketingPermissions(bool $marketing_permissions): void
    {
        $this->marketing_permissions = $marketing_permissions;
    }

    /**
     * @return string[]
     */
    public function getModules(): array
    {
        return $this->modules ?? [];
    }

    /**
     * @return MailingList\Stats
     */
    public function getStats(): MailingList\Stats
    {
        if (null === $this->stats) {
            $this->stats = new MailingList\Stats();
        }

        return $this->stats;
    }
}