<?php
namespace Ak\Mailchimp\Gateway;

use GuzzleHttp\ClientInterface;
use Laminas\Hydrator\HydratorInterface;

/**
 * Osnovna klasa za mailchimp gatewaye.
 *
 * @package Ak\Mailchimp\Gateway
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/autokreso/mailchimp
 */
abstract class AbstractGateway
{
    /** @var ClientInterface */
    protected $http;
    /** @var HydratorInterface */
    protected $hydrator;

    /**
     * Konstruktor.
     *
     * @param ClientInterface $http
     * @param HydratorInterface $hydrator
     */
    public function __construct(ClientInterface $http, HydratorInterface $hydrator)
    {
        $this->http = $http;
        $this->hydrator = $hydrator;
    }

    /** @noinspection PhpDocMissingThrowsInspection */

    /**
     * Upućuje request prema mailchimpu i vraća dekodirani json odgovor.
     *
     * @param string $method
     * @param string $path
     *
     * @param array $data
     * @return null|array
     */
    protected function request(string $method, string $path, array $data = []): ?array
    {
        if (in_array($method, ['PUT', 'POST'])) {
            $options = [
                'json' => $data,
            ];
        }

        /** @noinspection PhpUnhandledExceptionInspection */
        $response = $this->http->request($method, $path, $options ?? []);
        $content = $response->getBody()->getContents();
        return json_decode($content, true);
    }
}
