<?php
namespace Ak\Mailchimp\Service;

use GuzzleHttp\Client;
use GuzzleHttp\ClientInterface;
use Interop\Container\ContainerInterface;
use Laminas\ServiceManager\Factory\FactoryInterface;

/**
 * Kreira novu instancu http klijenta za mailchimp.
 *
 * @package Ak\Mailchimp\Service
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/autokreso/mailchimp
 */
final class HttpClientFactory implements FactoryInterface
{
    /**
     * Create an object
     *
     * @param  ContainerInterface $container
     * @param  string $requestedName
     * @param  null|array $options
     *
     * @return ClientInterface
     */
    public function __invoke(ContainerInterface $container, $requestedName, array $options = null)
    {
        $config = $container->get('config')['mailchimp'] ?? [];
        $base_uri = sprintf('https://%s.api.mailchimp.com/3.0/', $config['data_center']);

        return new Client([
            'base_uri' => $base_uri,
            'http_errors' => false,
            'auth' => [$config['username'], $config['api_key']],
        ]);
    }
}
