<?php
namespace AkTest\Mailchimp\Entity;

use Ak\Mailchimp\Entity\MailingList;
use Ak\Mailchimp\Entity\MailingList\CampaignDefaults;
use Ak\Mailchimp\Entity\MailingList\Contact;
use Ak\Mailchimp\Entity\MailingList\Stats;
use AkTest\Mailchimp\AbstractTestCase;
use DateTime;
use Zend\Hydrator\HydratorInterface;
use Zend\Hydrator\HydratorPluginManager;
use Zend\ServiceManager\PluginManagerInterface;

/**
 * Class MailingListTest
 *
 * @package AkTest\Mailchimp\Entity
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/autokreso/mailchimp
 */
final class MailingListTest extends AbstractTestCase
{
    /**
     *
     */
    public function testHydratorCanExchangeEntities()
    {
        $container = $this->getContainer();
        /** @var PluginManagerInterface $hydrators */
        $hydrators = $container->get(HydratorPluginManager::class);
        /** @var HydratorInterface $hydrator */
        $hydrator = $hydrators->get(MailingList::class);

        $data = [
            'id' => '34fb22',
            'web_id' => 4234,
            'name' => 'Ime liste',
            'contact' => [
                'company' => 'Neka firma d.o.o.',
                'address1' => 'Ulica i kućni broj',
                'address2' => 'Kat i stan',
                'city' => 'Grad',
                'state' => 'Županija',
                'zip' => 'Poštanski broj',
                'country' => 'Država',
                'phone' => 'Telefonski broj',
            ],
            'permission_reminder' => 'Nekakav podsjetnik',
            'use_archive_bar' => true,
            'campaign_defaults' => [
                'from_name' => 'Neko ime',
                'from_email' => 'korisnik@domena.tld',
                'subject' => 'Predmet',
                'language' => 'hrvatski',
            ],
            'notify_on_subscribe' => 'Obavijest za prijavu',
            'notify_on_unsubscribe' => 'Obavijest za odjavu',
            'date_created' => '2018-09-24T14:34:45+0200',
            'list_rating' => 543,
            'email_type_option' => true,
            'subscribe_url_short' => 'url-za-prijavu',
            'subscribe_url_long' => 'dugi-url-za-prijavu',
            'beamer_address' => 'beamer@domain.tld',
            'visibility' => 'pub',
            'double_optin' => true,
            'has_welcome' => true,
            'marketing_permissions' => true,
            'modules' => ['mod1', 'mod2'],
            'stats' => [
                'member_count' => 123,
                'unsubscribe_count' => 5632,
                'cleaned_count' => 24,
                'member_count_since_send' => 62,
                'unsubscribe_count_since_send' => 51,
                'cleaned_count_since_send' => 254,
                'campaign_count' => 235,
                'campaign_last_sent' => '2018-02-17T14:24:45+0200',
                'merge_field_count' => 146,
                'avg_sub_rate' => 13.5423,
                'avg_unsub_rate' => 15.342,
                'target_sub_rate' => 123.5345,
                'open_rate' => 154.234,
                'click_rate' => 134.234,
                'last_sub_date' => '2018-10-01T07:54:44+0200',
                'last_unsub_date' => '2018-10-02T21:12:45+0200',
            ],
        ];

        /** @var MailingList $list */
        $list = $hydrator->hydrate($data, new MailingList());

        $this->assertEquals($data['id'], $list->getId());
        $this->assertEquals($data['web_id'], $list->getWebId());
        $this->assertEquals($data['name'], $list->getName());

        $contact = $list->getContact();

        $this->assertInstanceOf(Contact::class, $contact);
        $this->assertEquals($data['contact']['company'], $contact->getCompany());
        $this->assertEquals($data['contact']['address1'], $contact->getAddress1());
        $this->assertEquals($data['contact']['address2'], $contact->getAddress2());
        $this->assertEquals($data['contact']['city'], $contact->getCity());
        $this->assertEquals($data['contact']['state'], $contact->getState());
        $this->assertEquals($data['contact']['zip'], $contact->getZip());
        $this->assertEquals($data['contact']['country'], $contact->getCountry());
        $this->assertEquals($data['contact']['phone'], $contact->getPhone());

        $this->assertEquals($data['permission_reminder'], $list->getPermissionReminder());
        $this->assertEquals($data['use_archive_bar'], $list->getUseArchiveBar());

        $defaults = $list->getCampaignDefaults();

        $this->assertInstanceOf(CampaignDefaults::class, $defaults);
        $this->assertEquals($data['campaign_defaults']['from_name'], $defaults->getFromName());
        $this->assertEquals($data['campaign_defaults']['from_email'], $defaults->getFromEmail());
        $this->assertEquals($data['campaign_defaults']['subject'], $defaults->getSubject());
        $this->assertEquals($data['campaign_defaults']['language'], $defaults->getLanguage());

        $this->assertEquals($data['notify_on_subscribe'], $list->getNotifyOnSubscribe());
        $this->assertEquals($data['notify_on_unsubscribe'], $list->getNotifyOnUnsubscribe());
        $this->assertEquals($data['date_created'], $list->getDateCreated()->format(DateTime::ISO8601));
        $this->assertEquals($data['list_rating'], $list->getListRating());
        $this->assertEquals($data['email_type_option'], $list->getEmailTypeOption());
        $this->assertEquals($data['subscribe_url_short'], $list->getSubscribeUrlShort());
        $this->assertEquals($data['subscribe_url_long'], $list->getSubscribeUrlLong());
        $this->assertEquals($data['beamer_address'], $list->getBeamerAddress());
        $this->assertEquals($data['visibility'], $list->getVisibility());
        $this->assertEquals($data['double_optin'], $list->getDoubleOptin());
        $this->assertEquals($data['has_welcome'], $list->getHasWelcome());
        $this->assertEquals($data['marketing_permissions'], $list->getMarketingPermissions());
        $this->assertEquals($data['modules'], $list->getModules());

        $stats = $list->getStats();

        $this->assertInstanceOf(Stats::class, $stats);
        $this->assertEquals($data['stats']['member_count'], $stats->getMemberCount());
        $this->assertEquals($data['stats']['unsubscribe_count'], $stats->getUnsubscribeCount());
        $this->assertEquals($data['stats']['cleaned_count'], $stats->getCleanedCount());
        $this->assertEquals($data['stats']['member_count_since_send'], $stats->getMemberCountSinceSend());
        $this->assertEquals($data['stats']['unsubscribe_count_since_send'], $stats->getUnsubscribeCountSinceSend());
        $this->assertEquals($data['stats']['cleaned_count'], $stats->getCleanedCount());
        $this->assertEquals($data['stats']['campaign_count'], $stats->getCampaignCount());
        $this->assertEquals($data['stats']['campaign_last_sent'], $stats->getCampaignLastSent()->format(DateTime::ISO8601));
        $this->assertEquals($data['stats']['merge_field_count'], $stats->getMergeFieldCount());
        $this->assertEquals($data['stats']['avg_sub_rate'], $stats->getAvgSubRate());
        $this->assertEquals($data['stats']['avg_unsub_rate'], $stats->getAvgUnsubRate());
        $this->assertEquals($data['stats']['target_sub_rate'], $stats->getTargetSubRate());
        $this->assertEquals($data['stats']['open_rate'], $stats->getOpenRate());
        $this->assertEquals($data['stats']['click_rate'], $stats->getClickRate());
        $this->assertEquals($data['stats']['last_sub_date'], $stats->getLastSubDate()->format(DateTime::ISO8601));
        $this->assertEquals($data['stats']['last_unsub_date'], $stats->getLastUnsubDate()->format(DateTime::ISO8601));

        $extracted = $hydrator->extract($list);

        ksort($data);
        ksort($extracted);

        $this->assertEquals($data, $extracted);
    }
}