<?php
namespace AkTest\Mailchimp\Response;

use Ak\Mailchimp\Entity\MailingList;
use Ak\Mailchimp\Response\GetListsResponse;
use AkTest\Mailchimp\AbstractTestCase;
use Zend\Hydrator\HydratorInterface;
use Zend\Hydrator\HydratorPluginManager;
use Zend\ServiceManager\PluginManagerInterface;

/**
 * Class GetListsResponseTest
 *
 * @package AkTest\Mailchimp\Response
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/autokreso/mailchimp
 */
final class GetListsTest extends AbstractTestCase
{
    public function testHydratorCanExchangeResponse()
    {
        $container = $this->getContainer();
        /** @var PluginManagerInterface $hydrators */
        $hydrators = $container->get(HydratorPluginManager::class);
        /** @var HydratorInterface $hydrator */
        $hydrator = $hydrators->get(GetListsResponse::class);

        $data = [
            'lists' => [[
                'id' => 'abc123',
            ], [
                'id' => 'cba321',
            ]],
            'total_items' => 8,
        ];

        /** @var GetListsResponse $response */
        $response = $hydrator->hydrate($data, new GetListsResponse());

        $this->assertCount(count($data['lists']), $response->getLists());

        $this->assertInstanceOf(MailingList::class, $response->getLists()[0]);
        $this->assertInstanceOf(MailingList::class, $response->getLists()[1]);
        $this->assertEquals($data['lists'][0]['id'], $response->getLists()[0]->getId());
        $this->assertEquals($data['lists'][1]['id'], $response->getLists()[1]->getId());
        $this->assertEquals($data['total_items'], $response->getTotalItems());

        $extracted = $hydrator->extract($response);

        $this->assertEquals(array_keys($data), array_keys($extracted));
        $this->assertCount(count($data['lists']), $extracted['lists']);
        $this->assertEquals($data['lists'][0]['id'], $extracted['lists'][0]['id']);
        $this->assertEquals($data['lists'][1]['id'], $extracted['lists'][1]['id']);
        $this->assertEquals($data['total_items'], $extracted['total_items']);
    }
}